@@ -1463,7 +1463,7 @@
 	} else if(model$modeldesc$distribution == "mvlaplace"){
 		if(length(rseed) == 1){
 			set.seed( rseed )
-			tmp = rugarch:::rged(m * (n.sim + n.start) * m.sim, 0, 1, shape = 1)
+			tmp = matrix(rugarch:::rged(m * (n.sim + n.start) * m.sim, 0, 1, shape = 1), , ncol = m, nrow = n.sim+n.start)
 			z = array(NA,  dim = c(n.sim + n.start + mo, m, m.sim))
 			for(i in 1:m.sim) z[,,i] = rbind(preZ, tmp)
 		} else{
@@ -1476,14 +1476,14 @@
 	} else{
 		if(length(rseed) == 1){
 			set.seed( rseed )
-			tmp = rugarch:::rstd(m * (n.sim + n.start) * m.sim, 0, 1, shape = model$mpars["mshape",1])
+			tmp = matrix(rugarch:::rstd(m * (n.sim + n.start) * m.sim, 0, 1, shape = model$pars["mshape",1]), ncol = m, nrow = n.sim+n.start)
 			z = array(NA,  dim = c(n.sim + n.start + mo, m, m.sim))
 			for(i in 1:m.sim) z[,,i] = rbind(preZ, tmp)
 		} else{
 			z = array(NA, dim = c(n.sim + n.start + mo, m, m.sim))
 			for(i in 1:m.sim){
 				set.seed( rseed[i] )
-				z[,,i] = rbind(preZ, matrix(rugarch:::rstd(m * (n.sim + n.start), 0, 1, shape = model$mpars["mshape",1]), nrow = n.sim + n.start, ncol = m))
+				z[,,i] = rbind(preZ, matrix(rugarch:::rstd(m * (n.sim + n.start), 0, 1, shape = model$pars["mshape",1]), nrow = n.sim + n.start, ncol = m))
 			}
 		}
 	}
